import 'dart:async';
import 'package:flutter/services.dart';

typedef void OnPlacementRequestSuccessListener();
typedef void OnPlacementRequestFailureListener(String error);
typedef void OnContentReadyListener();
typedef void OnContentShowListener();
typedef void OnContentDismissListener();
typedef void OnClickListener();
typedef void OnRewardRequestListener(int quantity);
typedef void OnTapjoyConnectSuccessListener();
typedef void OnTapjoyConnectFailureListener();

class Tapjoy {
  static Tapjoy get instance => _instance;
  final MethodChannel _channel;

  static final Tapjoy _instance = Tapjoy.private(
    const MethodChannel('native'),
  );

  Tapjoy.private(MethodChannel channel) : _channel = channel {
    _channel.setMethodCallHandler(_platformCallHandler);
  }

  static OnPlacementRequestSuccessListener _onPlacementRequestSuccessListener;
  static OnPlacementRequestFailureListener _onPlacementRequestFailureistener;
  static OnContentReadyListener _onContentReadyListener;
  static OnContentShowListener _onContentShowListener;
  static OnContentDismissListener _onContentDismissListener;
  static OnRewardRequestListener _onRewardRequestListener;
  static OnClickListener _onClickListener;
  static OnTapjoyConnectSuccessListener _onTapjoyConnectSuccessListener;
  static OnTapjoyConnectFailureListener _onTapjoyConnectFailureListener;

  Future<void> setDebugEnabled({bool isDebug}) async {
    assert(isDebug != null);
    return _channel.invokeMethod("setDebugEnabled", <String, dynamic>{
      'isDebug': isDebug,
    });
  }

  Future<void> setUserConsent({String consent}) async {
    assert(consent != null && consent.isNotEmpty);
    return _channel.invokeMethod("setUserConsent", <String, dynamic>{
      'consent': consent,
    });
  }

  Future<void> connect({String apiToken}) async {
    assert(apiToken != null && apiToken.isNotEmpty);
    return _channel.invokeMethod("connect", <String, dynamic>{
      'api_token': apiToken,
    });
  }

  Future<void> getPlacement({String placementName}) async {
    assert(placementName != null && placementName.isNotEmpty);
    return _channel.invokeMethod("getPlacement", <String, dynamic>{
      'placementName': placementName,
    });
  }

  Future<void> requestContent() async {
    try {
      await _channel.invokeMethod("requestContent");
    } catch (e) {
      print(e.toString());
    }
  }

  Future<void> showContent() async {
    try {
      await _channel.invokeMethod("showContent");
    } catch (e) {
      print(e.toString());
    }
  }

  Future _platformCallHandler(MethodCall call) async {
    switch (call.method) {
      case "onPlacementRequestSuccess":
        _onPlacementRequestSuccessListener();
        print("Tapjoy: onPlacementRequestSuccess");

        break;

      case "onPlacementRequestFailure":
        _onPlacementRequestFailureistener(call.arguments);
        print("Tapjoy : onPlacementRequestFailure");

        break;

      case "onContentReady":
        _onContentReadyListener();
        print("Tapjoy : onContentReady");

        break;

      case "onContentShow":
        _onContentShowListener();
        print("Tapjoy : onContentShow");

        break;
      case "onContentDismiss":
        _onContentDismissListener();
        print("Tapjoy : onContentDismiss");

        break;
      case "onRewardRequest":
        _onRewardRequestListener(call.arguments);
        print("Tapjoy : onRewardRequest");

        break;
      case "onClick":
        _onClickListener();
        print("Tapjoy : onClick");

        break;
      case "onTapjoyConnectSuccess":
        _onTapjoyConnectSuccessListener();
        print("Tapjoy : onTapjoyConnectSuccess");

        break;
      case "onTapjoyConnectFailure":
        _onTapjoyConnectFailureListener();
        print("Tapjoy : onTapjoyConnectFailure");

        break;
      default:
        print('Unknown method ${call.method} ');
    }
  }

  void setonPlacementRequestSuccessListener(
          OnPlacementRequestSuccessListener
              onPlacementRequestSuccessListener) =>
      _onPlacementRequestSuccessListener = onPlacementRequestSuccessListener;

  void setonPlacementRequestFailureListener(
          OnPlacementRequestFailureListener onPlacementRequestFailureistener) =>
      _onPlacementRequestFailureistener = onPlacementRequestFailureistener;

  void setonContentReadyListener(
          OnContentReadyListener onContentReadyListener) =>
      _onContentReadyListener = onContentReadyListener;

  void setonContentShowListener(OnContentShowListener onContentShowListener) =>
      _onContentShowListener = onContentShowListener;

  void setonContentDismissListener(
          OnContentDismissListener onContentDismissListener) =>
      _onContentDismissListener = onContentDismissListener;

  void setonRewardRequestListener(
          OnRewardRequestListener onRewardRequestListener) =>
      _onRewardRequestListener = onRewardRequestListener;

  void setOnClickListener(OnClickListener onClickListener) =>
      _onClickListener = onClickListener;

  void setOnTapjoyConnectFailureListener(
          OnTapjoyConnectFailureListener onTapjoyConnectFailureListener) =>
      _onTapjoyConnectFailureListener = onTapjoyConnectFailureListener;

  void setOnTapjoyConnectSuccessListener(
          OnTapjoyConnectSuccessListener onTapjoyConnectSuccessListener) =>
      _onTapjoyConnectSuccessListener = onTapjoyConnectSuccessListener;
}
